package com.flyfish.framework.controller;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.controller.reactive.ReactiveTreeController;
import com.flyfish.framework.domain.AdminUserDetails;
import com.flyfish.framework.domain.DepartmentQo;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.utils.UserUtils;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Mono;

import java.util.List;

/**
 * 部分或者校区controller
 */
@RestController
@RequestMapping("/departments")
public class DepartmentController extends ReactiveTreeController<Department, DepartmentQo> {

    /**
     * 获取有权限的tree
     *
     * @return 结果
     */
    @GetMapping("authorized-tree")
    public Mono<Result<List<Department>>> getAuthorizedTree() {
        return ReactiveSecurityContextHolder.getContext()
                .map(UserUtils::extractUserDetails)
                .flatMap(detail -> {
                    AdminUserDetails details = (AdminUserDetails) detail;
                    return reactiveService.getByIds(details.getAuthorityCodes()).collectList();
                })
                .map(this::makeTree)
                .map(Result::accept);
    }
}
